/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "DumpState.h"
#include <QMenu>
#include "Debugger.h"
#include "State.h"
#include "Instruction.h"
#include "DebuggerCoreInterface.h"
#include "Util.h"

#include <iostream>
#include <iomanip>
#include <string>
#include <sstream>


//------------------------------------------------------------------------------
// Name: DumpState(
// Desc: 
//------------------------------------------------------------------------------
DumpState::DumpState() : m_Menu(0) {
}

//------------------------------------------------------------------------------
// Name: ~DumpState()
// Desc: 
//------------------------------------------------------------------------------
DumpState::~DumpState() {
	delete m_Menu;
}

//------------------------------------------------------------------------------
// Name: 
//------------------------------------------------------------------------------
template<typename T>
std::string OutputHexString(T value) {
	std::stringstream ss;
	ss << std::hex << std::setw(sizeof(T) * 2) << std::setfill('0') <<  static_cast<unsigned long>(value);
	return ss.str();
}

//------------------------------------------------------------------------------
// Name: menu(QWidget *parent)
// Desc: 
//------------------------------------------------------------------------------
QMenu *DumpState::menu(QWidget *parent) {

	if(m_Menu == 0) {
		m_Menu = new QMenu("DumpState", parent);
		m_Menu->addAction ("&Dump Current State", this, SLOT(doMenu()), QKeySequence("Ctrl+D"));
	}
	
	return m_Menu;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void DumpState::dumpCode(const State &state) {
	edb::address_t address = state.eip;
	for(int i = 0; i < 6; ++i) {
		uint8_t buf[Instruction::MAX_SIZE];
		int size = sizeof(buf);
		
		if(Debugger::getInstructionBytes(address, buf, size)) {
			Instruction insn(buf, size, address, std::nothrow);
			if(insn.valid()) {
				std::cout << OutputHexString(address) << ": " << insn.format() << "\n";
			} else {
				break;
			}
			address += insn.size();
		} else {
			break;
		}
		
	}
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void DumpState::dumpRegisters(const State &state) {
	std::cout << "     eax:" << OutputHexString(state.eax);
	std::cout << " ebx:" << OutputHexString(state.ebx);
	std::cout << "  ecx:" << OutputHexString(state.ecx);
	std::cout << "  edx:" << OutputHexString(state.edx);
	std::cout << "     eflags:" << OutputHexString(state.eflags);
	std::cout << "\n";
	std::cout << "     esi:" << OutputHexString(state.esi);
	std::cout << " edi:" << OutputHexString(state.edi);
	std::cout << "  esp:" << OutputHexString(state.esp);
	std::cout << "  ebp:" << OutputHexString(state.ebp);
	std::cout << "     eip:" << OutputHexString(state.eip);
	std::cout << "\n";
	std::cout << "     cs:" << OutputHexString(state.cs);
	std::cout << "  ds:" << OutputHexString(state.ds);
	std::cout << "  es:" << OutputHexString(state.es);
	std::cout << "  fs:" << OutputHexString(state.fs);
	std::cout << "  gs:" << OutputHexString(state.gs);
	std::cout << "  ss:" << OutputHexString(state.ss);
	std::cout << "    ";
	std::cout << ((state.eflags & (1 << 11)) != 0 ? 'O' : 'o') << ' ';
	std::cout << ((state.eflags & (1 << 10)) != 0 ? 'D' : 'd') << ' ';
	std::cout << ((state.eflags & (1 <<  9)) != 0 ? 'I' : 'i') << ' ';
	std::cout << ((state.eflags & (1 <<  8)) != 0 ? 'T' : 't') << ' ';
	std::cout << ((state.eflags & (1 <<  7)) != 0 ? 'S' : 's') << ' ';
	std::cout << ((state.eflags & (1 <<  6)) != 0 ? 'Z' : 'z') << ' ';
	std::cout << ((state.eflags & (1 <<  4)) != 0 ? 'A' : 'a') << ' ';
	std::cout << ((state.eflags & (1 <<  2)) != 0 ? 'P' : 'p') << ' ';
	std::cout << ((state.eflags & (1 <<  0)) != 0 ? 'C' : 'c');
	std::cout << "\n";
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void DumpState::dumpLines(edb::address_t address, int lines) {
	for(int i = 0; i < lines; ++i) {
		uint8_t buf[16];
		if(Debugger::debuggerBase->readBytes(address, buf, sizeof(buf))) {
			std::cout << OutputHexString(address) << " : ";
			
			for(int j = 0x00; j < 0x04; ++j) std::cout << OutputHexString(buf[j]) << " ";
			std::cout << " ";
			for(int j = 0x04; j < 0x08; ++j) std::cout << OutputHexString(buf[j]) << " ";
			std::cout << "- ";
			for(int j = 0x08; j < 0x0c; ++j) std::cout << OutputHexString(buf[j]) << " ";
			std::cout << " ";
			for(int j = 0x0c; j < 0x10; ++j) std::cout << OutputHexString(buf[j]) << " ";

			for(int j = 0; j < 16; ++j) std::cout << (safe_ctype<std::isprint>(buf[j]) ? static_cast<char>(buf[j]) : '.');
					
			std::cout << "\n";
		} else {
			break;
		}
		address += 16;
	}
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void DumpState::dumpStack(const State &state) {
	dumpLines(state.esp, 4);
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void DumpState::dumpData(const State &state) {
	dumpLines(state.esi, 2);
}

//------------------------------------------------------------------------------
// Name: doMenu()
// Desc: 
//------------------------------------------------------------------------------
void DumpState::doMenu() {

	State &state = Debugger::currentState();

	std::cout << "------------------------------------------------------------------------------\n";
	dumpRegisters(state);
	std::cout << "[" << OutputHexString(state.ss) << ":" << OutputHexString(state.esp) << "]---------------------------------------------------------[stack]\n";
	dumpStack(state);
	std::cout << "[" << OutputHexString(state.ds) << ":" << OutputHexString(state.esi) << "]---------------------------------------------------------[ data]\n";
	dumpData(state);
	std::cout << "[" << OutputHexString(state.cs) << ":" << OutputHexString(state.eip) << "]---------------------------------------------------------[ code]\n";
	dumpCode(state);
	std::cout << "------------------------------------------------------------------------------\n";
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
QString DumpState::author() const {
	return "Evan Teran";
}

Q_EXPORT_PLUGIN2(DumpState, DumpState)
