/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "DialogStrings.h"
#include "DebuggerCoreInterface.h"
#include "Debugger.h"
#include "MemoryRegions.h"
#include "Util.h"
#include "Configuration.h"

#include <QHeaderView>
#include <QMessageBox>
#include <QVector>
#include <QSortFilterProxyModel>

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
DialogStrings::DialogStrings(QWidget *parent) : QDialog(parent) {
	ui.setupUi(this);
	ui.tableView->verticalHeader()->hide();
	ui.tableView->horizontalHeader()->setStretchLastSection(true);
	
	m_FilterModel = new QSortFilterProxyModel(this);
	connect(ui.txtSearch, SIGNAL(textChanged(const QString &)), m_FilterModel, SLOT(setFilterFixedString(const QString &)));
}

//------------------------------------------------------------------------------
// Name: on_listWidget_itemDoubleClicked(QListWidgetItem *item)
// Desc: follows the found item in the data view
//------------------------------------------------------------------------------
void DialogStrings::on_listWidget_itemDoubleClicked(QListWidgetItem *item) {
	bool ok;
	const QString s = item->text();
	// TODO: remove hard coded 8
	const edb::address_t addr = s.left(8).toUInt(&ok, 16);
	if(ok) {
		Debugger::dumpData(addr);
	}
}


//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void DialogStrings::showEvent(QShowEvent *) {
	m_FilterModel->setFilterKeyColumn(3);
	m_FilterModel->setSourceModel(&Debugger::memoryRegions());
	ui.tableView->setModel(m_FilterModel);
	
	ui.progressBar->setValue(0);
	ui.listWidget->clear();
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void DialogStrings::on_btnFind_clicked() {
	const QItemSelectionModel *const selModel = ui.tableView->selectionModel();
	const QModelIndexList sel = selModel->selectedRows();

	edb::address_t start_address	= 0;
	edb::address_t end_address	= 0;
	
	QString str;
	ui.listWidget->clear();

	if(sel.size() == 0) {
		QMessageBox::information(this, "No Region Selected", "You must select a region which is to be scanned for strings.");
	}

	foreach(const QModelIndex selected_item, sel) {

		
		
		const QModelIndex index = m_FilterModel->mapToSource(selected_item);
		const MemoryRegions::Region *const region = reinterpret_cast<const MemoryRegions::Region *>(index.internalPointer());
		
		ui.progressBar->setValue(0);
		if(region != 0) {

			start_address	= region->start;
			end_address		= region->end;

			const edb::address_t orig_start	= start_address;
			const edb::address_t howMany		= end_address - orig_start;

			// do the search for this region!
			while(start_address < end_address) {

				int stringLen = 0;		
				const bool ok = Debugger::getASCIIStringAtAddress(start_address, str, Debugger::config()->minAsciiStringLength, INT_MAX, stringLen);
				if(ok) {
					ui.listWidget->addItem(QString().sprintf("%08x: %s", start_address, qPrintable(str)));
				}

				ui.progressBar->setValue(CalculatePercentage((start_address - orig_start), howMany));

				if(ok) {				
					start_address += stringLen;
				} else {
					++start_address;
				}
			}
		}
		ui.progressBar->setValue(100);
	}
}
