/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "CommentServer.h"
#include "Instruction.h"
#include "Configuration.h"
#include "Debugger.h"
#include "DebuggerCoreInterface.h"
#include <QString>

//------------------------------------------------------------------------------
// Name: CommentServer()
// Desc: 
//------------------------------------------------------------------------------
CommentServer::CommentServer() {
}

//------------------------------------------------------------------------------
// Name: setComment(edb::address_t address, const QString &comment)
// Desc: 
//------------------------------------------------------------------------------
void CommentServer::setComment(edb::address_t address, const QString &comment) {
	Q_UNUSED(address);
	Q_UNUSED(comment);
}

// a call can be anyhere from 2 to 7 bytes long depends on if there is a Mod/RM byte
// or a SIB byte, etc
// this is ignoring prefixes, fortunately, no calls have mandatory prefixes
#define CALL_MAX_SIZE 7
#define CALL_MIN_SIZE 2

//------------------------------------------------------------------------------
// Name: resolveFunctionCall(edb::address_t address) const
// Desc: 
//------------------------------------------------------------------------------
QString CommentServer::resolveFunctionCall(edb::address_t address) const {
	QString ret;
	
	edb::reg_t value;
	// get the value that address points to (usually [esp+x])
	if(Debugger::debuggerBase->readBytes(address, &value, sizeof(value))) {
	
		// ok, we now want to locate the instruction before this one
		// so we need to look back a few bytes
		uint8_t buffer[Instruction::MAX_SIZE];
		
		// TODO: portability warning, makes assumptions on the size of a call
		// TODO: don't make direct use of OP_CALL test, this is implementation of Instruction
		//       specific
		if(Debugger::debuggerBase->readBytes(value - CALL_MAX_SIZE, buffer, sizeof(buffer))) {
			for(int i = (CALL_MAX_SIZE - CALL_MIN_SIZE); i >= 0; --i) {
				Instruction insn(buffer + i, sizeof(buffer) - i, 0, std::nothrow);
				if(insn.valid() && insn.type() == Instruction::OP_CALL) {
					const QString symname = Debugger::findFunctionSymbol(value, "");
					if(!symname.isEmpty()) {
						ret = QString().sprintf("Return to %08x <%s>", value, qPrintable(symname));
					} else {
						ret = QString().sprintf("Return to %08x", value);
					}
					break;
				}
			}
		}
	}
	return ret;
}

//------------------------------------------------------------------------------
// Name: resolveAsciiString(edb::address_t address) const
// Desc: 
//------------------------------------------------------------------------------
QString CommentServer::resolveAsciiString(edb::address_t address) const {
	QString ret;
	QString temp;
	
	edb::reg_t value;
	if(Debugger::debuggerBase->readBytes(address, &value, sizeof(value))) {
		int stringLen;
		if(Debugger::getASCIIStringAtAddress(value, temp, Debugger::config()->minAsciiStringLength, 256, stringLen)) {
			ret = "ASCII \"" + temp + "\"";
		}
	}
	return ret;
}

//------------------------------------------------------------------------------
// Name: getComment(edb::address_t address) const
// Desc: 
//------------------------------------------------------------------------------
QString CommentServer::getComment(edb::address_t address) const {

	QString ret;
	
	ret = resolveFunctionCall(address);
	if(ret.isEmpty()) {
		ret = resolveAsciiString(address);
	}
	
	return ret;
}

