/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "ELF32BinaryInfo.h"
#include <QFile>

//------------------------------------------------------------------------------
// Name: ELF32BinaryInfo(const QString &filename)
// Desc: constructor
//------------------------------------------------------------------------------
ELF32BinaryInfo::ELF32BinaryInfo(const QString &filename) : BinaryInfo(filename), m_Header(0) {

}

//------------------------------------------------------------------------------
// Name: ~ELF32BinaryInfo()
// Desc: deconstructor
//------------------------------------------------------------------------------
ELF32BinaryInfo::~ELF32BinaryInfo() {
	delete m_Header;
	m_Header = 0;
}

//------------------------------------------------------------------------------
// Name: validateHeader()
// Desc: returns true if this file matches this particular info class
//------------------------------------------------------------------------------
bool ELF32BinaryInfo::validateHeader() {
	readHeader();
	if(std::memcmp(m_Header->e_ident, ELFMAG, SELFMAG) == 0) {
		return m_Header->e_ident[EI_CLASS] == ELFCLASS32;
	}
	return false;
}

//------------------------------------------------------------------------------
// Name: getEntryPoint()
// Desc: returns the entry point if any of the binary
//------------------------------------------------------------------------------
edb::address_t ELF32BinaryInfo::getEntryPoint() {
	readHeader();
	return m_Header->e_entry;
}

//------------------------------------------------------------------------------
// Name: readHeader()
// Desc: reads in enough of the file to get the header
//------------------------------------------------------------------------------
void ELF32BinaryInfo::readHeader() {
	if(m_Header == 0) {
		m_Header = new Elf32_Ehdr;
		
		QFile file(m_Filename);
		file.open(QIODevice::ReadOnly);
		if(file.isOpen()) {
			file.read(reinterpret_cast<char *>(m_Header), sizeof(Elf32_Ehdr));
		}
	}
}
