/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef MD5_20070328_H_
#define MD5_20070328_H_

#include <cstddef>
#include <stdint.h>
#include <climits>

class MD5 {
public:
	MD5();
	
private:
	void init();
	
public:
	void update(const void *input, std::size_t input_length);
	void finalize();
	const uint8_t *digest() const;
	
private:
	uint8_t		m_Buffer[64];	// input buffer
	uint32_t	m_State[4];		// current digest
	uint64_t	m_Count64;		// number of bits
	bool		m_Finalized;	// have we been finalized

private:
	void transform(const uint8_t buffer[64]);
	
private:
	template <typename T> 
	static T rol(T v, int n) {
		static const std::size_t bits(CHAR_BIT * sizeof(T));
		const T mask(~(T(-1) << n));
		return (v << n) | ((v >> bits - n) & mask);
	}
	
	template<uint32_t (*Fun)(uint32_t, uint32_t, uint32_t)>
	static void doTransform(uint32_t& a, uint32_t b, uint32_t c, uint32_t d, uint32_t x, uint32_t  s, uint32_t ac) {
		a += Fun(b, c, d) + x + ac;
		a = rol(a, s) + b;
	}
	
private:						
	static uint32_t F(uint32_t x, uint32_t y, uint32_t z);
	static uint32_t G(uint32_t x, uint32_t y, uint32_t z);
	static uint32_t H(uint32_t x, uint32_t y, uint32_t z);
	static uint32_t I(uint32_t x, uint32_t y, uint32_t z);
};

#endif

