
#include "Debugger.h"
#include "DebuggerCoreInterface.h"
#include <QFileInfo>
#include <QTextStream>

//------------------------------------------------------------------------------
// Name: symlinkTarget(const QFileInfo &info)
// Desc: 
//------------------------------------------------------------------------------
QString Debugger::symlinkTarget(const QFileInfo &info) {
#if QT_VERSION < 0x040200
	QString ret;
	#if defined(Q_OS_LINUX) || defined(Q_OS_FREEBSD)
		char buf[PATH_MAX];
		const ssize_t i = readlink(qPrintable(info.absoluteFilePath()), buf, sizeof(buf));
		if(i >= 0) {
			buf[i] = '\0';
			ret = buf;
		}
	#else
		#error "Unsupported Platform"
	#endif
	return ret;
#else
	return info.symLinkTarget();
#endif
}

//------------------------------------------------------------------------------
// Name: getProcessCurrentWorkingDirectory()
// Desc: 
//------------------------------------------------------------------------------
QString Debugger::getProcessCurrentWorkingDirectory() {
	QString ret;
	if(debuggerBase != 0) {
		const edb::pid_t pid = debuggerBase->pid();
	#ifdef Q_OS_LINUX
		ret = Debugger::symlinkTarget(QString().sprintf("/proc/%d/cwd", pid));
	#else
		#error "Unsupported Platform"
	#endif
	}
	return ret;
}

//------------------------------------------------------------------------------
// Name: getProcessArguments()
// Desc: 
//------------------------------------------------------------------------------
QStringList Debugger::getProcessArguments() {
	QStringList ret;
	if(debuggerBase != 0) {
		const edb::pid_t pid = debuggerBase->pid();
	#ifdef Q_OS_LINUX
		const QString cmdlineFile(QString().sprintf("/proc/%d/cmdline", pid));
		QFile file(cmdlineFile);

		if(file.open(QIODevice::ReadOnly | QIODevice::Text)) {
			QTextStream in(&file);

			QString s;
			QChar ch;

			while(in.status() == QTextStream::Ok) {
				in >> ch;
				if(ch == '\0') {
					if(!s.isEmpty()) {
						ret.append(s);
					}
					s.clear();
				} else {
					s += ch;
				}
			}

			if(!s.isEmpty()) {
				ret.append(s);
			}
		}
	#else
		#error "Unsupported Platform"
	#endif
	}
	return ret;
}
